(AJS.toInit(function($) {
    // Autoconvert only available in Confluence 4.1 onwards
    if (!tinymce.plugins.Autoconvert)
        return;

    /**
     * Helper function - determines if the _file_ part of a URI represents the filename of an Office document, by
     * examining the file extension.
     * @param filename The filename to check (may be null or undefined).
     * @returns true if the filename is an Office document; false otherwise.
     */
    function isOfficeFilename(filename) {
        if (!filename || filename == "")
            return false;

        var dotPos = filename.indexOf(".");
        if (dotPos < 0)
            return false;

        var extension = filename.substring(dotPos + 1).toLowerCase();
        var validExtensions = ["doc", "docx", "xls", "xlsx", "ppt", "pptx"];
        var isValid;
        $.each(validExtensions, function(index, validExtension) {
            if (extension == validExtension) {
                isValid = true;
                return false;
            }
        });
        return isValid;
    }

    /**
     * Register an Autoconvert listener to convert URLs to SharePoint documents to be converted into sp-link macro
     * instances when the URL is pasted into the Confluence editor.
     * @param uri A parsed URI object representing the URL that was pasted into the editor. See http://stevenlevithan.com/demo/parseuri/js/
     * @param node The DOM node that was pasted.
     * @param done Callback to execute when this handler has executed. Done must _always_ be called.
     */
    tinymce.plugins.Autoconvert.autoConvert.addHandler(function(uri, node, done) {
        // Try to match the pasted URI to one of the SharePoint sites configured in Confluence. If the pasted URI does
        // not start with the authority of one of the SharePoint sites, we will not match against it.
        AJS.SharePoint.getSitesAsync(function(sites) {
            var existingUri = decodeURI(uri.source); // handle special characters in URI

            var match = SharePoint.getClosestMatch(sites, uri);
            if (!match) {
                AJS.log("SharePoint link autoconvert aborting because the link does not match a configured SharePoint site " + uri.source);
                done(); // no match
                return;
            }

            // Ensure that the pasted URI links to an Office Document hosted in SharePoint (ie. has a .docx, .ppt, etc. extension)
            // It could just be a link to a SharePoint site, list or other managed path, in which case the macro would
            // serve no purpose.
            if (!isOfficeFilename(uri.file)) {
                AJS.log("SharePoint link autoconvert aborting because the link is not an Office Document " + uri.source);
                done();
                return;
            }

            // Ok, convert to macro definition and insert into the DOM.
            AJS.log("SharePoint link autoconvert converting URI to macro " + uri.source);
            macro = {
                name: "sp-link",
                params: {
                    url: existingUri,
                    title: decodeURI(uri.file)
                }
            };
            tinymce.plugins.Autoconvert.convertMacroToDom(macro, done, done);
        }, function(jqXhr, textStatus, errorThrown) {
            AJS.log("SharePoint link autoconvert aborted because the list of configured SharePoint sites could not be retrieved from Confluence " + errorThrown);
            done();
        });
    });
}))(AJS.$);
